/**
 *        Copyright (c) 1993-2006 IONA Technologies PLC.
 *                       All Rights Reserved.
 */

package demo.asset.server;

import java.io.File;
import java.util.logging.Logger;

import com.iona.asset.types.Employee;
import com.iona.asset.types.InvalidId;
import com.iona.asset.types.Vehicle;
import com.iona.asset.CompanyAssetPortType;
import com.iona.asset.GetEmployeeFault;
import com.iona.asset.GetVehicleFault;

@javax.jws.WebService(portName = "CompanyAssetPortType",
                      serviceName = "CompanyAssetService",
                      targetNamespace = "http://iona.com/asset", 
                      endpointInterface = "com.iona.asset.CompanyAssetPortType")
                  
public class CompanyAssetImpl implements CompanyAssetPortType {

    private static final Logger LOG = 
        Logger.getLogger(CompanyAssetImpl.class.getPackage().getName());

    private DatabaseConnector db;

    CompanyAssetImpl(String dbLocation) {
        System.out.println("dbLocation " + dbLocation);
        if (dbLocation == null || "".equals(dbLocation)) {
            db = new DatabaseConnector("build" + File.separator + "var" 
                + File.separator + "assetDb");
        } else {
            db = new DatabaseConnector(dbLocation);
        }
    }

    public Employee getEmployee(int id) throws GetEmployeeFault {
        LOG.info("Executing operation getEmployee " +  id);
        System.out.println("Executing operation getEmployee " + id + "\n");
        Employee e = db.getEmployee(id);
        if (e == null) {
            InvalidId invalid = new InvalidId();
            invalid.setDetails("No Employee with id: " + id);
            throw new GetEmployeeFault("Error getting employee", invalid);
        }
        return e;
    }

    public void addEmployee(Employee e) {
        db.addEmployee(e);
    }
    
    public void removeEmployee(int id) {
        db.removeEmployee(id);
    }
    
    public Vehicle getVehicle(int id) throws GetVehicleFault {
        LOG.info("Executing operation getVehicle " +  id);
        System.out.println("Executing operation getVehicle " + id + "\n");
        Vehicle v = db.getVehicle(id);
        if (v == null) {
            InvalidId invalid = new InvalidId();
            invalid.setDetails("No Vehicle with id: " + id);
            throw new GetVehicleFault("Error getting vehicle", invalid);
        }
        return v;
    }
    
    public void addVehicle(Vehicle v) {
        db.addVehicle(v);
    }
    
    public void removeVehicle(int id) {
        db.removeVehicle(id);
    }
}
