/**
 *        Copyright (c) 1993-2006 IONA Technologies PLC.
 *                       All Rights Reserved.
 */

package demo.asset.server;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.util.Properties;
import java.util.Random;
import java.util.logging.Logger;

import com.iona.asset.types.Employee;
import com.iona.asset.types.Vehicle;

public class DatabaseConnector {

    private static final Logger LOG = 
        Logger.getLogger(DatabaseConnector.class.getPackage().getName());
        
    private static final String framework = "embedded";
    private static final String driver = "org.apache.derby.jdbc.EmbeddedDriver";
    private static final String protocol = "jdbc:derby:";

    private Connection connection;
    private Statement statement;

    DatabaseConnector(String dbName) {
        connect(dbName);
    }
    
    public Employee getEmployee(int id) {
        Employee e = null;

        String query = "SELECT name, license FROM employee WHERE id=" + id;
        try {
            ResultSet rs = statement.executeQuery(query);
            if (rs.next()) {
                e = new Employee();
                e.setId(id);
                e.setName(rs.getString(1));
                e.setLicense(rs.getString(2));
            }
            rs.close();
        } catch (SQLException ex) {
            System.err.println("Exception getting employee result: " + ex.toString());
        }

        return e;
    }

    public void addEmployee(Employee e) {
        String query = "INSERT INTO employee VALUES (" + e.getId() + ", '" 
            + e.getName() + "', '" + e.getLicense() + "')";
        try {
            statement.execute(query);
        } catch (SQLException ex) {
            System.err.println("Exception adding vehicle: " + ex.toString());
        }
    }
    
    public void removeEmployee(int id) {
        String query = "DELETE FROM employee WHERE id=" + id;
        try {
            statement.execute(query);
        } catch (SQLException ex) {
            System.err.println("Exception deleting vehicle: " + ex.toString());
        }
    }
    
    public Vehicle getVehicle(int id) {
        Vehicle v = null;

        String query = "SELECT capacity FROM vehicle WHERE id=" + id;
        try {
            ResultSet rs = statement.executeQuery(query);
            if (rs.next()) {
                v = new Vehicle();
                v.setId(id);
                v.setCapacity(rs.getInt(1));
            }
            rs.close();
        } catch (SQLException ex) {
            System.err.println("Exception getting vehicle result: " + ex.toString());
        }

        return v;
    }

    public void addVehicle(Vehicle v) {
        String query = "INSERT INTO vehicle VALUES (" + v.getId() + ", " 
            + v.getCapacity() + ")";
        try {
            statement.execute(query);
        } catch (SQLException ex) {
            System.err.println("Exception adding vehicle: " + ex.toString());
        }
    }
    
    public void removeVehicle(int id) {
        String query = "DELETE FROM vehicle WHERE id=" + id;
        try {
            statement.execute(query);
        } catch (SQLException ex) {
            System.err.println("Exception deleting vehicle: " + ex.toString());
        }
    }
    
    private void setupTables() {
        System.out.println("adding data to the created database");
        String[] data = new String[] {
            "CREATE TABLE employee(id INT NOT NULL, name VARCHAR(32), license VARCHAR(16))",
            "CREATE TABLE vehicle(id INT NOT NULL, capacity INT)"
        };
        try {
            for (String s : data) {
                statement.execute(s);
            }
        } catch (SQLException ex) {
            System.err.println("Exception executing statement " + ex.toString());
        }
        // Just some data for when the database is created.
        // Probably be better to load this stuff from a file
        String[][] employees = new String[][] {
            { "Mr. McFeely", "19700101fred" },
            { "Doctor Who", "20060203timelord" },
            { "Rose Tyler", "20060405abcdefgh" },
            { "Bobby Orr", "20060404bruin" },
            { "Dalek", "20060607mutant" },
            { "Patrick McGoohan", "19280809NaN" },
            { "Terry Pratchett", "19750505ankh" },
            { "JarJar Binks", "19990518naboo" }
        };
        Random random = new Random();
        int id = 0;
        for (String[] e : employees) {
            Employee employee = new Employee();
            employee.setId(id);
            employee.setName(e[0]);
            employee.setLicense(e[1]);
            addEmployee(employee);
            
            Vehicle v = new Vehicle();
            v.setId(id);
            v.setCapacity(random.nextInt(64000));
            addVehicle(v);
            
            id++;
        }
    }

    private void connect(String dbName) {
        try {
            Class.forName(driver).newInstance();
        } catch (Exception ex) {
            System.err.println("Exception instantiating derby embedded driver. " 
                + ex.toString());
            return;
        }
        Properties props = new Properties();
        props.put("user", "user1");
        props.put("password", "user1");
       
        ShutdownHook shutdownHook = new ShutdownHook();
        Runtime.getRuntime().addShutdownHook(shutdownHook);

        try {
            connection = DriverManager.getConnection(protocol + dbName, props);
            statement = connection.createStatement();
            connection.setAutoCommit(false);
            System.out.println("Connected to database assetDB");
        } catch (SQLException ex1) {
            try {
                connection = DriverManager.getConnection(protocol + dbName
                    + ";create=true", props);
                statement = connection.createStatement();
                connection.setAutoCommit(false);
                setupTables();
                System.out.println("Created database assetDB");
            } catch (SQLException ex2) {
                System.err.println("Exception creating database " 
                    + ex2.toString());
            }
        }
    }

    private void shutdown() {
        try {
            System.out.println("DatabaseConnector.shutdown");
            if (statement != null) {
                statement.close();
                statement = null;
            }
            if (connection != null) {
                connection.commit();
                connection.close();
                connection = null;
            }
        } catch (SQLException ex) {
            System.err.println("Exception closing database connection " 
                + ex.toString());
        }

        try {
            DriverManager.getConnection(protocol + ";shutdown=true");
        } catch (SQLException se) {
            System.out.println("Database shut down normally");
        }
    }
    
    private class ShutdownHook extends Thread {
        public void run() {
            shutdown();
        }
    }
}

