/**
 *        Copyright (c) 1993-2006 IONA Technologies PLC.
 *                       All Rights Reserved.
 */

package demo.location.client;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import javax.xml.namespace.QName;
import com.iona.location_types.Coordinates;
import com.iona.location_types.Location;
import com.iona.location_updater.LocationUpdaterService;
import com.iona.location_updater.LocationUpdaterPortType;

public final class Client {

    private static final QName SERVICE_NAME 
        = new QName("http://iona.com/location_updater", "LocationUpdaterService");
    private static final int NUM_EMPLOYEES = 8;
    private static final int STEP_COUNT = 4;
    private static final int STEP_INTERVAL = 5000;

    // Some destinations to iterate through - a
    // straight line between each destination (latitude,
    // longitude) pair, follows a street on the map.
    private static final double[][] destinations = {
        { 40.7141671548, -74.006385379 },
        { 40.7149321268, -74.0079466267 },
        { 40.7194900955, -74.0043581958 },
        { 40.7186911165, -74.002523427 }
    };
    
    private Client() {
    } 

    public static void main(String args[]) throws Exception {
        
        if (args.length == 0) { 
            System.out.println("please specify wsdl");
            System.exit(1); 
        }

        URL wsdlURL;
        File wsdlFile = new File(args[0]);
        if (wsdlFile.exists()) {
            wsdlURL = wsdlFile.toURL();
        } else {
            wsdlURL = new URL(args[0]);
        }
        
        System.out.println(wsdlURL);
        LocationUpdaterService ls = new LocationUpdaterService(wsdlURL, SERVICE_NAME);
        LocationUpdaterPortType port = ls.getLocationUpdaterRouterPort();

        List<Coordinates> coordinates;
        coordinates = new ArrayList<Coordinates>();
        for (double[] d : destinations ) {
            Coordinates c = new Coordinates();
            System.out.println(d[0] + " " + d[1]);
            c.setLatitude(d[0]);
            c.setLongitude(d[1]);
            coordinates.add(c);
        }
        Location l = new Location();
        Random r = new Random();
        l.setVehicleId(r.nextInt(NUM_EMPLOYEES));
        l.setEmployeeId(r.nextInt(NUM_EMPLOYEES));
        // Send a location update every STEP_INTERVAL milliseconds.
        int index = 0;
        int nextIndex = index + 1;
        if (nextIndex >= coordinates.size()) {
            System.err.println("Destination list must have at least 2 entries");
            System.exit(0); 
        }
        // Run through the destination list forever...
        while (true) {
            Coordinates current = new Coordinates();
            current.setLatitude(coordinates.get(index).getLatitude());
            current.setLongitude(coordinates.get(index).getLongitude());
            l.setCoordinates(current);
            System.out.println("Coordinates prior to update : lat=" + l.getCoordinates().getLatitude() + " long=" + l.getCoordinates().getLongitude());
            port.updateLocation(l);
            try {
                Thread.sleep(STEP_INTERVAL);
            } catch (java.lang.InterruptedException ex) {
                // oh well.
            }
            Coordinates next = coordinates.get(nextIndex);
            double currentLat = current.getLatitude();
            double currentLong = current.getLongitude();
            double latDelta = (next.getLatitude() - currentLat) / STEP_COUNT;
            double longDelta = (next.getLongitude() - currentLong) / STEP_COUNT;
            for (int i = 0; i < STEP_COUNT; i++) {
                currentLat += latDelta;
                currentLong += longDelta;
                current.setLatitude(currentLat);
                current.setLongitude(currentLong);
                System.out.println("Update location: " + currentLat + " " 
                    + currentLong);
                l.setCoordinates(current);
                port.updateLocation(l);
                try {
                    Thread.sleep(STEP_INTERVAL);
                } catch (java.lang.InterruptedException ex) {
                    // oh well.
                }
            }
            index = nextIndex;
            nextIndex++;
            if (nextIndex >= coordinates.size()) {
                System.out.println("next verse... same as the first");
                nextIndex = 0;
            }
        }
    }

}
